@@ -1,7 +1,7 @@
 # -- coding: utf-8 --
 # MIT License
 #
-# Copyright (c) 2017-2019 Charles Jekel
+# Copyright (c) 2017-2022 Charles Jekel
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -35,7 +35,7 @@ from pyDOE import lhs
 class PiecewiseLinFit(object):
 
     def __init__(self, x, y, disp_res=False, lapack_driver='gelsd', degree=1,
-                 weights=None):
+                 weights=None, seed=None):
         r"""
         An object to fit a continuous piecewise linear function
         to data.
@@ -70,6 +70,11 @@ class PiecewiseLinFit(object):
             standard deviation for each data point, where weights[i]
             corresponds to one over the standard deviation of the ith data
             point. Default weights=None.
+        seed : None, or int
+            Pick an integer which will set the numpy.random.seed on init.
+            The fit and fitfast methods rely on stochastic methods and setting
+            this value will make the results reproducible. The default
+            behavior is to not specify a seed.
 
         Attributes
         ----------
@@ -105,6 +110,8 @@ class PiecewiseLinFit(object):
         se : ndarray (1-D)
             Standard errors associated with each beta parameter. Specifically
             se[0] correspounds to the standard error for beta[0], and so forth.
+        seed : int
+            Numpy random seed number set on init.
         slopes : ndarray (1-D)
             The slope of each ling segment as a 1-D numpy array. This assumes
             that x[0] <= x[1] <= ... <= x[n]. Thus, slopes[0] is the slope
@@ -212,6 +219,10 @@ class PiecewiseLinFit(object):
             # self.weights2 = weights*weights
             self.y_w = np.dot(self.y_data, np.eye(self.n_data) * self.weights)
 
+        if seed:
+            np.random.seed(seed)
+        self.seed = seed
+
         # initialize all empty attributes as None
         self.fit_breaks = None
         self.n_segments = None
@@ -265,6 +276,12 @@ class PiecewiseLinFit(object):
         b_[0], b_[1:-1], b_[-1] = self.break_0, v.copy(), self.break_n
         return b_
 
+    def _fit_one_segment(self):
+        r"""
+        Fit for a single line segment
+        """
+        self.fit_with_breaks([self.break_0, self.break_n])
+
     def assemble_regression_matrix(self, breaks, x):
         r"""
         Assemble the linear regression matrix A
@@ -570,7 +587,7 @@ class PiecewiseLinFit(object):
         # try to solve the regression problem
         try:
             # least squares solver
-            ssr = self.lstsq(A)
+            ssr = self.lstsq(A, calc_slopes=False)
 
         except linalg.LinAlgError:
             # the computation could not converge!
@@ -628,7 +645,7 @@ class PiecewiseLinFit(object):
         breaks = breaks[breaks_order]
 
         A = self.assemble_regression_matrix(breaks, self.x_data)
-        L = self.conlstsq(A)
+        L = self.conlstsq(A, calc_slopes=False)
         return L
 
     def fit(self, n_segments, x_c=None, y_c=None, bounds=None, **kwargs):
@@ -738,6 +755,11 @@ class PiecewiseLinFit(object):
         # calculate the number of variables I have to solve for
         self.nVar = self.n_segments - 1
 
+        # special fit for one line segment
+        if self.n_segments == 1:
+            self._fit_one_segment()
+            return self.fit_breaks
+
         # initiate the bounds of the optimization
         if bounds is None:
             bounds = np.zeros([self.nVar, 2])
@@ -849,6 +871,11 @@ class PiecewiseLinFit(object):
         # calculate the number of variables I have to solve for
         self.nVar = self.n_segments - 1
 
+        # special fit for one line segment
+        if self.n_segments == 1:
+            self._fit_one_segment()
+            return self.fit_breaks
+
         # initiate the bounds of the optimization
         if bounds is None:
             bounds = np.zeros([self.nVar, 2])
@@ -1207,11 +1234,11 @@ class PiecewiseLinFit(object):
             variance = np.dot(e, e) / (ny - nb)
             if self.weights is None:
                 # solve for the unbiased estimate of variance
-                A2inv = np.abs(linalg.inv(np.dot(A.T, A)).diagonal())
+                A2inv = np.abs(linalg.pinv(np.dot(A.T, A)).diagonal())
                 self.se = np.sqrt(variance * A2inv)
             else:
                 A = (A.T*self.weights).T
-                A2inv = np.abs(linalg.inv(np.dot(A.T, A)).diagonal())
+                A2inv = np.abs(linalg.pinv(np.dot(A.T, A)).diagonal())
                 self.se = np.sqrt(variance * A2inv)
             return self.se
 
@@ -1297,7 +1324,7 @@ class PiecewiseLinFit(object):
         # try to solve for the prediction variance at the x locations
         try:
             pre_var = variance * \
-                np.dot(np.dot(A, linalg.inv(np.dot(Ad.T, Ad))), A.T)
+                np.dot(np.dot(A, linalg.pinv(np.dot(Ad.T, Ad))), A.T)
             return pre_var.diagonal()
 
         except linalg.LinAlgError:
@@ -1447,9 +1474,18 @@ class PiecewiseLinFit(object):
         p = 2.0 * stats.t.sf(np.abs(t), df=n - k - 1)
         return p
 
-    def lstsq(self, A):
+    def lstsq(self, A, calc_slopes=True):
         r"""
         Perform the least squares fit for A matrix.
+
+        Parameters
+        ----------
+        A : ndarray (2-D)
+            The regression matrix you want to fit in the linear system of
+            equations Ab=y.
+        calc_slopes : boolean, optional
+            Whether to calculate slopes after performing a fit. Default is
+            calc_slopes=True.
         """
         if self.weights is None:
             beta, ssr, _, _ = linalg.lstsq(A, self.y_data,
@@ -1481,13 +1517,23 @@ class PiecewiseLinFit(object):
         # save the beta parameters
         self.beta = beta
 
-        # save the slopes
-        self.calc_slopes()
+        if calc_slopes:
+            # save the slopes
+            self.calc_slopes()
         return ssr
 
-    def conlstsq(self, A):
+    def conlstsq(self, A, calc_slopes=True):
         r"""
         Perform a constrained least squares fit for A matrix.
+
+        Parameters
+        ----------
+        A : ndarray (2-D)
+            The regression matrix you want to fit in the linear system of
+            equations Ab=y.
+        calc_slopes : boolean, optional
+            Whether to calculate slopes after performing a fit. Default is
+            calc_slopes=True.
         """
         # Assemble the constraint matrix
         C_list = [np.ones_like(self.x_c)]
@@ -1538,7 +1584,8 @@ class PiecewiseLinFit(object):
             self.zeta = beta_prime[self.n_parameters:]
 
             # save the slopes
-            self.calc_slopes()
+            if calc_slopes:
+                self.calc_slopes()
 
             # Calculate ssr
             # where ssr = sum of square of residuals
